<?php

namespace App\Service;

use App\Aggregate\AcceptOrderCommand;
use App\Exception\ClientLockedException;
use App\Exception\OrderInvalidException;
use Symfony\Component\Serializer\SerializerInterface;
use Symfony\Component\Validator\Validator\ValidatorInterface;
use Symfony\Contracts\HttpClient\HttpClientInterface;

readonly class OrderService
{
    public function __construct(
        private HttpClientInterface $crmClient,
        private SerializerInterface $serializer,
        private PricingStrategy $pricingStrategy,
        private ClientService $clientService,
        private ValidatorInterface $validator,
    ) {
    }

    /**
     * @throws \App\Exception\ClientLockedException
     * @throws \App\Exception\OrderInvalidException
     */
    public function acceptOrder(AcceptOrderCommand $command)
    {
        $violations = $this->validator->validate($command);

        if ($violations->count() > 0) {
            throw OrderInvalidException::createFromViolations($command->order, $violations);
        }

        if ($command->client->isLocked()) {
            throw new ClientLockedException(sprintf("Client '%s' is locked.", $command->client->getName()));
        }

        $this->crmClient->request('POST', '/order', [
            'body' => $this->serializer->serialize($command->order, format: 'json')
        ]);

        $totalPrice = $command->order->getTotalPrice($this->pricingStrategy);

        $this->clientService->deduceFromBalance($command->client, $totalPrice);
    }
}
